@@ -9,51 +9,67 @@ Utility module for dealing with colormaps.
 import json
 import math
 import os
+from typing import Dict, List, Optional, Sequence, Tuple, Union
 
 from jinja2 import Template
 
 from branca.element import ENV, Figure, JavascriptLink, MacroElement
 from branca.utilities import legend_scaler
 
-rootpath = os.path.abspath(os.path.dirname(__file__))
+rootpath: str = os.path.abspath(os.path.dirname(__file__))
 
 with open(os.path.join(rootpath, "_cnames.json")) as f:
-    _cnames = json.loads(f.read())
+    _cnames: Dict[str, str] = json.loads(f.read())
 
 with open(os.path.join(rootpath, "_schemes.json")) as f:
-    _schemes = json.loads(f.read())
+    _schemes: Dict[str, List[str]] = json.loads(f.read())
 
 
-def _is_hex(x):
+TypeRGBInts = Tuple[int, int, int]
+TypeRGBFloats = Tuple[float, float, float]
+TypeRGBAInts = Tuple[int, int, int, int]
+TypeRGBAFloats = Tuple[float, float, float, float]
+TypeAnyColorType = Union[TypeRGBInts, TypeRGBFloats, TypeRGBAInts, TypeRGBAFloats, str]
+
+
+def _is_hex(x: str) -> bool:
     return x.startswith("#") and len(x) == 7
 
 
-def _parse_hex(color_code):
+def _parse_hex(color_code: str) -> TypeRGBAFloats:
     return (
-        int(color_code[1:3], 16),
-        int(color_code[3:5], 16),
-        int(color_code[5:7], 16),
+        _color_int_to_float(int(color_code[1:3], 16)),
+        _color_int_to_float(int(color_code[3:5], 16)),
+        _color_int_to_float(int(color_code[5:7], 16)),
+        1.0,
     )
 
 
-def _parse_color(x):
+def _color_int_to_float(x: int) -> float:
+    """Convert an integer between 0 and 255 to a float between 0. and 1.0"""
+    return x / 255.0
+
+
+def _color_float_to_int(x: float) -> int:
+    """Convert a float between 0. and 1.0 to an integer between 0 and 255"""
+    return int(x * 255.9999)
+
+
+def _parse_color(x: Union[tuple, list, str]) -> TypeRGBAFloats:
     if isinstance(x, (tuple, list)):
-        color_tuple = tuple(x)[:4]
-    elif isinstance(x, (str, bytes)) and _is_hex(x):
-        color_tuple = _parse_hex(x)
-    elif isinstance(x, (str, bytes)):
+        return tuple(tuple(x) + (1.0,))[:4]  # type: ignore
+    elif isinstance(x, str) and _is_hex(x):
+        return _parse_hex(x)
+    elif isinstance(x, str):
         cname = _cnames.get(x.lower(), None)
         if cname is None:
             raise ValueError(f"Unknown color {cname!r}.")
-        color_tuple = _parse_hex(cname)
+        return _parse_hex(cname)
     else:
         raise ValueError(f"Unrecognized color code {x!r}")
-    if max(color_tuple) > 1.0:
-        color_tuple = tuple(u / 255.0 for u in color_tuple)
-    return tuple(map(float, (color_tuple + (1.0,))[:4]))
 
 
-def _base(x):
+def _base(x: float) -> float:
     if x > 0:
         base = pow(10, math.floor(math.log10(x)))
         return round(x / base) * base
@@ -72,22 +88,32 @@ class ColorMap(MacroElement):
         The right bound of the color scale.
     caption: str
         A caption to draw with the colormap.
+    text_color: str, default "black"
+        The color for the text.
     max_labels : int, default 10
         Maximum number of legend tick labels
     """
 
-    _template = ENV.get_template("color_scale.js")
+    _template: Template = ENV.get_template("color_scale.js")
 
-    def __init__(self, vmin=0.0, vmax=1.0, caption="", max_labels=10):
+    def __init__(
+        self,
+        vmin: float = 0.0,
+        vmax: float = 1.0,
+        caption: str = "",
+        text_color: str = "black",
+        max_labels: int = 10,
+    ):
         super().__init__()
         self._name = "ColorMap"
 
         self.vmin = vmin
         self.vmax = vmax
         self.caption = caption
-        self.index = [vmin, vmax]
+        self.text_color = text_color
+        self.index: List[float] = [vmin, vmax]
         self.max_labels = max_labels
-        self.tick_labels = None
+        self.tick_labels: Optional[Sequence[Union[float, str]]] = None
 
         self.width = 450
         self.height = 40
@@ -95,9 +121,13 @@ class ColorMap(MacroElement):
     def render(self, **kwargs):
         """Renders the HTML representation of the element."""
         self.color_domain = [
-            self.vmin + (self.vmax - self.vmin) * k / 499.0 for k in range(500)
+            float(self.vmin + (self.vmax - self.vmin) * k / 499.0) for k in range(500)
         ]
         self.color_range = [self.__call__(x) for x in self.color_domain]
+
+        # sanitize possible numpy floats to native python floats
+        self.index = [float(i) for i in self.index]
+
         if self.tick_labels is None:
             self.tick_labels = legend_scaler(self.index, self.max_labels)
 
@@ -113,7 +143,7 @@ class ColorMap(MacroElement):
             name="d3",
         )  # noqa
 
-    def rgba_floats_tuple(self, x):
+    def rgba_floats_tuple(self, x: float) -> TypeRGBAFloats:
         """
         This class has to be implemented for each class inheriting from
         Colormap. This has to be a function of the form float ->
@@ -123,37 +153,37 @@ class ColorMap(MacroElement):
         """
         raise NotImplementedError
 
-    def rgba_bytes_tuple(self, x):
+    def rgba_bytes_tuple(self, x: float) -> TypeRGBAInts:
         """Provides the color corresponding to value `x` in the
         form of a tuple (R,G,B,A) with int values between 0 and 255.
         """
-        return tuple(int(u * 255.9999) for u in self.rgba_floats_tuple(x))
+        return tuple(_color_float_to_int(u) for u in self.rgba_floats_tuple(x))  # type: ignore
 
-    def rgb_bytes_tuple(self, x):
+    def rgb_bytes_tuple(self, x: float) -> TypeRGBInts:
         """Provides the color corresponding to value `x` in the
         form of a tuple (R,G,B) with int values between 0 and 255.
         """
         return self.rgba_bytes_tuple(x)[:3]
 
-    def rgb_hex_str(self, x):
+    def rgb_hex_str(self, x: float) -> str:
         """Provides the color corresponding to value `x` in the
         form of a string of hexadecimal values "#RRGGBB".
         """
         return "#%02x%02x%02x" % self.rgb_bytes_tuple(x)
 
-    def rgba_hex_str(self, x):
+    def rgba_hex_str(self, x: float) -> str:
         """Provides the color corresponding to value `x` in the
         form of a string of hexadecimal values "#RRGGBBAA".
         """
         return "#%02x%02x%02x%02x" % self.rgba_bytes_tuple(x)
 
-    def __call__(self, x):
+    def __call__(self, x: float) -> str:
         """Provides the color corresponding to value `x` in the
         form of a string of hexadecimal values "#RRGGBBAA".
         """
         return self.rgba_hex_str(x)
 
-    def _repr_html_(self):
+    def _repr_html_(self) -> str:
         """Display the colormap in a Jupyter Notebook.
 
         Does not support all the class arguments.
@@ -181,22 +211,32 @@ class ColorMap(MacroElement):
                     for i in range(self.width)
                 ],
             )
-            + '<text x="0" y="38" style="text-anchor:start; font-size:11px; font:Arial">{}</text>'.format(  # noqa
+            + (
+                '<text x="0" y="38" style="text-anchor:start; font-size:11px;'
+                ' font:Arial; fill:{}">{}</text>'
+            ).format(
+                self.text_color,
                 self.vmin,
             )
             + "".join(
                 [
                     (
-                        '<text x="{}" y="38"; style="text-anchor:middle; font-size:11px; font:Arial">{}</text>'  # noqa
-                    ).format(x_ticks[i], val_ticks[i])
+                        '<text x="{}" y="38"; style="text-anchor:middle; font-size:11px;'
+                        ' font:Arial; fill:{}">{}</text>'
+                    ).format(x_ticks[i], self.text_color, val_ticks[i])
                     for i in range(1, nb_ticks - 1)
                 ],
             )
-            + '<text x="{}" y="38" style="text-anchor:end; font-size:11px; font:Arial">{}</text>'.format(
+            + (
+                '<text x="{}" y="38" style="text-anchor:end; font-size:11px;'
+                ' font:Arial; fill:{}">{}</text>'
+            ).format(
                 self.width,
+                self.text_color,
                 self.vmax,
             )
-            + '<text x="0" y="12" style="font-size:11px; font:Arial">{}</text>'.format(
+            + '<text x="0" y="12" style="font-size:11px; font:Arial; fill:{}">{}</text>'.format(
+                self.text_color,
                 self.caption,
             )
             + "</svg>"
@@ -229,6 +269,10 @@ class LinearColormap(ColorMap):
     vmax : float, default 1.
         The maximal value for the colormap.
         Values higher than `vmax` will be bound directly to `colors[-1]`.
+    caption: str
+        A caption to draw with the colormap.
+    text_color: str, default "black"
+        The color for the text.
     max_labels : int, default 10
         Maximum number of legend tick labels
     tick_labels: list of floats, default None
@@ -236,21 +280,23 @@ class LinearColormap(ColorMap):
 
     def __init__(
         self,
-        colors,
-        index=None,
-        vmin=0.0,
-        vmax=1.0,
-        caption="",
-        max_labels=10,
-        tick_labels=None,
+        colors: Sequence[TypeAnyColorType],
+        index: Optional[Sequence[float]] = None,
+        vmin: float = 0.0,
+        vmax: float = 1.0,
+        caption: str = "",
+        text_color: str = "black",
+        max_labels: int = 10,
+        tick_labels: Optional[Sequence[float]] = None,
     ):
         super().__init__(
             vmin=vmin,
             vmax=vmax,
             caption=caption,
+            text_color=text_color,
             max_labels=max_labels,
         )
-        self.tick_labels = tick_labels
+        self.tick_labels: Optional[Sequence[float]] = tick_labels
 
         n = len(colors)
         if n < 2:
@@ -259,9 +305,9 @@ class LinearColormap(ColorMap):
             self.index = [vmin + (vmax - vmin) * i * 1.0 / (n - 1) for i in range(n)]
         else:
             self.index = list(index)
-        self.colors = [_parse_color(x) for x in colors]
+        self.colors: List[TypeRGBAFloats] = [_parse_color(x) for x in colors]
 
-    def rgba_floats_tuple(self, x):
+    def rgba_floats_tuple(self, x: float) -> TypeRGBAFloats:
         """Provides the color corresponding to value `x` in the
         form of a tuple (R,G,B,A) with float values between 0. and 1.
         """
@@ -278,20 +324,20 @@ class LinearColormap(ColorMap):
         else:
             raise ValueError("Thresholds are not sorted.")
 
-        return tuple(
+        return tuple(  # type: ignore
             (1.0 - p) * self.colors[i - 1][j] + p * self.colors[i][j] for j in range(4)
         )
 
     def to_step(
         self,
-        n=None,
-        index=None,
-        data=None,
-        method=None,
-        quantiles=None,
-        round_method=None,
-        max_labels=10,
-    ):
+        n: Optional[int] = None,
+        index: Optional[Sequence[float]] = None,
+        data: Optional[Sequence[float]] = None,
+        method: str = "linear",
+        quantiles: Optional[Sequence[float]] = None,
+        round_method: Optional[str] = None,
+        max_labels: int = 10,
+    ) -> "StepColormap":
         """Splits the LinearColormap into a StepColormap.
 
         Parameters
@@ -352,11 +398,7 @@ class LinearColormap(ColorMap):
                 max_ = max(data)
                 min_ = min(data)
                 scaled_cm = self.scale(vmin=min_, vmax=max_)
-                method = (
-                    "quantiles"
-                    if quantiles is not None
-                    else method if method is not None else "linear"
-                )
+                method = "quantiles" if quantiles is not None else method
                 if method.lower().startswith("lin"):
                     if n is None:
                         raise ValueError(msg)
@@ -411,6 +453,7 @@ class LinearColormap(ColorMap):
         ]
 
         caption = self.caption
+        text_color = self.text_color
 
         return StepColormap(
             colors,
@@ -418,11 +461,17 @@ class LinearColormap(ColorMap):
             vmin=index[0],
             vmax=index[-1],
             caption=caption,
+            text_color=text_color,
             max_labels=max_labels,
             tick_labels=self.tick_labels,
         )
 
-    def scale(self, vmin=0.0, vmax=1.0, max_labels=10):
+    def scale(
+        self,
+        vmin: float = 0.0,
+        vmax: float = 1.0,
+        max_labels: int = 10,
+    ) -> "LinearColormap":
         """Transforms the colorscale so that the minimal and maximal values
         fit the given parameters.
         """
@@ -435,6 +484,7 @@ class LinearColormap(ColorMap):
             vmin=vmin,
             vmax=vmax,
             caption=self.caption,
+            text_color=self.text_color,
             max_labels=max_labels,
         )
 
@@ -465,6 +515,10 @@ class StepColormap(ColorMap):
     vmax : float, default 1.
         The maximal value for the colormap.
         Values higher than `vmax` will be bound directly to `colors[-1]`.
+    caption: str
+        A caption to draw with the colormap.
+    text_color: str, default "black"
+        The color for the text.
     max_labels : int, default 10
         Maximum number of legend tick labels
     tick_labels: list of floats, default None
@@ -473,18 +527,20 @@ class StepColormap(ColorMap):
 
     def __init__(
         self,
-        colors,
-        index=None,
-        vmin=0.0,
-        vmax=1.0,
-        caption="",
-        max_labels=10,
-        tick_labels=None,
+        colors: Sequence[TypeAnyColorType],
+        index: Optional[Sequence[float]] = None,
+        vmin: float = 0.0,
+        vmax: float = 1.0,
+        caption: str = "",
+        text_color: str = "black",
+        max_labels: int = 10,
+        tick_labels: Optional[Sequence[float]] = None,
     ):
         super().__init__(
             vmin=vmin,
             vmax=vmax,
             caption=caption,
+            text_color=text_color,
             max_labels=max_labels,
         )
         self.tick_labels = tick_labels
@@ -496,9 +552,9 @@ class StepColormap(ColorMap):
             self.index = [vmin + (vmax - vmin) * i * 1.0 / n for i in range(n + 1)]
         else:
             self.index = list(index)
-        self.colors = [_parse_color(x) for x in colors]
+        self.colors: List[TypeRGBAFloats] = [_parse_color(x) for x in colors]
 
-    def rgba_floats_tuple(self, x):
+    def rgba_floats_tuple(self, x: float) -> TypeRGBAFloats:
         """
         Provides the color corresponding to value `x` in the
         form of a tuple (R,G,B,A) with float values between 0. and 1.
@@ -510,9 +566,13 @@ class StepColormap(ColorMap):
             return self.colors[-1]
 
         i = len([u for u in self.index if u <= x])  # 0 < i < n.
-        return tuple(self.colors[i - 1])
+        return self.colors[i - 1]
 
-    def to_linear(self, index=None, max_labels=10):
+    def to_linear(
+        self,
+        index: Optional[Sequence[float]] = None,
+        max_labels: int = 10,
+    ) -> LinearColormap:
         """
         Transforms the StepColormap into a LinearColormap.
 
@@ -540,10 +600,17 @@ class StepColormap(ColorMap):
             index=index,
             vmin=self.vmin,
             vmax=self.vmax,
+            caption=self.caption,
+            text_color=self.text_color,
             max_labels=max_labels,
         )
 
-    def scale(self, vmin=0.0, vmax=1.0, max_labels=10):
+    def scale(
+        self,
+        vmin: float = 0.0,
+        vmax: float = 1.0,
+        max_labels: int = 10,
+    ) -> "StepColormap":
         """Transforms the colorscale so that the minimal and maximal values
         fit the given parameters.
         """
@@ -556,6 +623,7 @@ class StepColormap(ColorMap):
             vmin=vmin,
             vmax=vmax,
             caption=self.caption,
+            text_color=self.text_color,
             max_labels=max_labels,
         )
 
@@ -569,7 +637,7 @@ class _LinearColormaps:
         for key, val in _schemes.items():
             setattr(self, key, LinearColormap(val))
 
-    def _repr_html_(self):
+    def _repr_html_(self) -> str:
         return Template(
             """
         <table>
@@ -592,7 +660,7 @@ class _StepColormaps:
         for key, val in _schemes.items():
             setattr(self, key, StepColormap(val))
 
-    def _repr_html_(self):
+    def _repr_html_(self) -> str:
         return Template(
             """
         <table>
